SUBROUTINE BCIMP( x, BCType, BotTop, CPHS, CSHS, rhoHS, F, G, IPow )                                                   

  ! Compute Boundary Condition IMPedance

  USE krakcmod
  USE RefCoMod

  IMPLICIT NONE
  INTEGER                :: Itop, Ibot, Medium
  INTEGER, INTENT( OUT ) :: IPow
  REAL          (KIND=8) :: omega, rhoHS, c0, rhoIns, RadDeg, thetaInt, RInt, PhiInt
  COMPLEX       (KIND=8) :: x, Kx, Kz, TWERSK, GamS2, GamP2, GamS, GamP, mu, &
       yV( 5 ), PEKRT, RCmplx, CPHS, CSHS, CInside
  COMPLEX (KIND=8), INTENT( OUT ) :: F, G
  CHARACTER BCType*1, BotTop*3

  IPow = 0 

  ! Get rho, C just INSide the boundary               
  ! (There is at least one acoustic layer in the problem, except
  !  in the case where BOUNCE is used to get the refl. coef. for       
  !  a purely elastic stack of layers.                                 

  IF ( BotTop(1:3) == 'TOP' ) THEN 
     IF ( FirstAcoustic > 0 ) THEN
        Itop    = Loc( FirstAcoustic ) + N( FirstAcoustic ) + 1
        rhoINS  = rho( Itop )
        CInside = PEKRT( Omega2 * H( FirstAcoustic ) **2 / ( 2.0 + B1( Itop ) ) )
     ENDIF
  ELSE 
     IF ( LastAcoustic > 0 ) THEN 
        Ibot    = Loc( LastAcoustic ) + N( LastAcoustic ) + 1 
        rhoINS  = rho( Ibot )
        CInside = PEKRT( Omega2 * H( LastAcoustic ) **2 / ( 2.0 + B1( Ibot ) ) )
     ENDIF
  ENDIF

  ! Vacuum
  IF ( BCType(1:1) == 'V' ) THEN 
     F = 1.0 
     !G = -i * PEKRT( Omega2 / CInside ** 2 - x ) * SIGMA( 1 ) ** 2
     G = 0.0
     yV( 1 ) = F
     yV( 2 ) = G
     yV( 3 ) = 0.0
     yV( 4 ) = 0.0
     yV( 5 ) = 0.0
  ENDIF

  ! Vacuum with Twersky scatter model
  IF ( BCType(1:1) == 'S' .OR. BCType(1:1) == 'H' .OR. BCType(1:1) == 'T' .OR. BCType(1:1) == 'I' ) THEN        
     Omega = SQRT( Omega2 ) 
     Kx = SQRT( x )
     F = 1.0
     C0 = REAL( CInside )
     G = TWERSK( BCType, Omega, BUMDEN, xI, ETA, Kx, rhoINS, C0 )
     G = G / ( i * Omega * rhoINS )
     yV( 1 ) = F
     yV( 2 ) = G
     yV( 3 ) = 0.0
     yV( 4 ) = 0.0
     yV( 5 ) = 0.0
  ENDIF

  ! Rigid
  IF ( BCType(1:1) == 'R' ) THEN 
     F = 0.0
     G = 1.0
     yV( 1 ) = F
     yV( 2 ) = G
     yV( 3 ) = 0.0
     yV( 4 ) = 0.0
     yV( 5 ) = 0.0
  ENDIF

  ! Acousto-elastic half-space
  IF ( BCType(1:1) == 'A' ) THEN 
     IF ( REAL( CSHS ) > 0.0 ) THEN 
        GamS2 = x - Omega2 / CSHS ** 2 
        GamP2 = x - Omega2 / CPHS ** 2
        GamS  = PEKRT( GamS2 )
        GamP  = PEKRT( GamP2 )

        mu = rhoHS * CSHS ** 2

        yV( 1 ) = ( GamS*GamP - x ) / mu 
        yV( 2 ) = ( ( GamS2 + x ) ** 2 - 4.0*GamS * GamP * x ) * mu
        yV( 3 ) = 2.0 * GamS * GamP - GamS2 - x
        yV( 4 ) = GamP * ( x - GamS2 )
        yV( 5 ) = GamS * ( GamS2 - x )

        F = Omega2 * yV( 4 )
        G = yV( 2 )
     ELSE 
        GamP = PEKRT( x - Omega2 / CPHS ** 2 ) 
        F = 1.0
        G = rhoHS / GamP 
     ENDIF
  ENDIF

  ! Tabulated reflection coefficient
  IF ( BCType(1:1) == 'F' ) THEN 
     ! Compute the grazing angle THETA
     Kx       = SQRT( x ) 
     Kz       = SQRT( Omega2 / CInside ** 2 - x ) 
     RadDeg   = 360.0 / ( 2.0 * PI )
     ThetaInt = RadDeg * DATAN2( DBLE( Kz ), DBLE( Kx ) )

     ! Evaluate R( ThetaInt )
     IF ( BotTop(1:3) == 'TOP' ) THEN
        CALL REFCO( ThetaInt, RInt, PhiInt, ThetaTop, RTop, PhiTop, NTopPts, PRTFil )
     ELSE
        CALL REFCO( ThetaInt, RInt, PhiInt, ThetaBot, RBot, PhiBot, NBotPts, PRTFil )
     ENDIF

     ! Convert R(THETA) to (f,g) in Robin BC
     RCmplx = RInt * EXP( i * PhiInt )
     F      = 1.0
     G      = ( 1.0 + RCmplx ) / ( i * Kz * ( 1.0 - RCmplx ) )
  ENDIF

  IF ( BotTop(1:3) == 'TOP' ) G = -G    ! A top BC has the sign flipped relative to a bottom BC

  ! Precalculated reflection coef
  IF ( BCType(1:1) == 'P' ) THEN 
     CALL IRCINT( x, F, G, IPow, xTab, FTab, GTab, ITab, NkTab ) 
  ENDIF

  ! Shoot through elastic layers
  IF ( BotTop(1:3) == 'TOP' ) THEN 
     IF ( FirstAcoustic > 1 ) THEN 
        DO Medium = 1, FirstAcoustic - 1   ! Shoot down from top
           CALL ELASDN( x, yV, IPow, Medium )
        END DO

        F = Omega2 * yV( 4 )
        G = yV( 2 )
     ENDIF
  ELSE 
     IF ( LastAcoustic < NMedia ) THEN
        DO Medium = NMedia, LastAcoustic + 1, -1  ! Shoot up from bottom
           CALL ELASUP( x, yV, IPow, Medium )
        END DO

        F = Omega2 * yV( 4 )
        G = yV( 2 )
     ENDIF
  ENDIF

  RETURN 
END SUBROUTINE BCIMP
!**********************************************************************C
SUBROUTINE ELASUP( x, yV, IPow, Medium ) 

  ! Propagates through an elastic layer using compound matrix formulation

  USE krakcmod

  IMPLICIT NONE
  INTEGER          :: IPowR, IPowF, IPow, Medium, II, J
  REAL    (KIND=8) :: Roof, Floor, Twoh
  COMPLEX (KIND=8) :: x, xV( 5 ), yV( 5 ), zV( 5 ), Twox, xB3, FourHx
  PARAMETER ( Roof = 1.0E5, Floor = 1.0E-5, IPowR = 5, IPowF = -5 )

  ! Euler's method for first step                                     

  Twox   = 2.0 * x 
  TwoH   = 2.0 * H( Medium ) 
  FourHx = 4.0 * H( Medium ) * x
  J      = Loc( Medium ) + N( Medium ) + 1
  xB3    = x * B3( J ) - rho( J )

  zV(1) = yV(1) - 0.5*(   B1( J ) * yV( 4 ) - B2( J ) * yV( 5 ) ) 
  zV(2) = yV(2) - 0.5*( -rho( J ) * yV( 4 ) -     xB3 * yV( 5 ) ) 
  zV(3) = yV(3) - 0.5*(      TwoH * yV( 4 ) + B4( J ) * yV( 5 ) )
  zV(4) = yV(4) - 0.5*(    xB3 * yV(1) + B2(J) * yV(2) -Twox * B4(J) * yV(3) ) 
  zV(5) = yV(5) - 0.5*( rho(J) * yV(1) - B1(J) * yV(2) -      FourHx * yV(3) ) 

  ! Modified midpoint method                                          
  DO II = N( Medium ), 1, -1 
     J = J - 1
     xV = yV 
     yV = zV 

     xB3 = x * B3( J ) - rho( J ) 

     zV(1) = xV(1) - (   B1( J ) * yV( 4 ) - B2( J ) * yV( 5 ) ) 
     zV(2) = xV(2) - ( -rho( J ) * yV( 4 ) -     xB3 * yV( 5 ) ) 
     zV(3) = xV(3) - (      TwoH * yV( 4 ) + B4( J ) * yV( 5 ) )
     zV(4) = xV(4) - (    xB3 * yV(1) + B2(J) * yV(2) - Twox * B4(J) * yV(3) ) 
     zV(5) = xV(5) - ( rho(J) * yV(1) - B1(J) * yV(2) -       FourHx * yV(3) ) 

     ! Scale if necessary
     IF ( II /= 1 ) THEN 
        IF ( ABS( DBLE( zV( 2 ) ) ) < Floor ) THEN 
           zV   = Roof * zV
           yV   = Roof * yV 
           IPow = IPow - IPowR 
        ENDIF

        IF ( ABS( DBLE( zV( 2 ) ) ) > Roof  ) THEN 
           zV   = Floor * zV
           yV   = Floor * yV 
           IPow = IPow - IPowF 
        ENDIF
     ENDIF
  END DO

  yV = ( xV + 2.0 * yV + zV ) / 4.0   ! Apply the standard filter at the terminal point

  RETURN 
END SUBROUTINE ELASUP
!**********************************************************************C
SUBROUTINE ELASDN( x, yV, IPow, Medium ) 

  ! Propagates through an elastic layer using compound matrix formulation

  USE krakcmod

  IMPLICIT NONE
  INTEGER          :: IPowR, IPowF, IPow, Medium, II, J
  REAL    (KIND=8) :: Roof, Floor, Twoh
  COMPLEX (KIND=8) :: x, xV( 5 ), yV( 5 ), zV( 5 ), Twox, xB3, FourHx 
  PARAMETER ( Roof = 1.0E5, Floor = 1.0E-5, IPowR = 5, IPowF = -5 )

  ! Euler's method for first step                                     

  Twox   = 2.0 * x 
  TwoH   = 2.0 * H( Medium ) 
  FourHx = 4.0 * H( Medium ) * x
  J      = Loc( Medium ) + 1
  xB3    = x * B3( J ) - rho( J )

  zV(1) = yV(1) + 0.5*(   B1( J ) * yV( 4 ) - B2( J ) * yV( 5 ) ) 
  zV(2) = yV(2) + 0.5*( -rho( J ) * yV( 4 ) -     xB3 * yV( 5 ) ) 
  zV(3) = yV(3) + 0.5*(      TwoH * yV( 4 ) + B4( J ) * yV( 5 ) )
  zV(4) = yV(4) + 0.5*(    xB3 * yV(1) + B2(J) * yV(2) -Twox * B4(J) * yV(3) ) 
  zV(5) = yV(5) + 0.5*( rho(J) * yV(1) - B1(J) * yV(2) -      FourHx * yV(3) ) 

  ! Modified midpoint method                                          
  DO II = 1, N( Medium ) 
     J = J + 1

     xV = yV 
     yV = zV 

     xB3 = x * B3( J ) - rho( J ) 

     zV(1) = xV(1) + (   B1( J ) * yV( 4 ) - B2( J ) * yV( 5 ) ) 
     zV(2) = xV(2) + ( -rho( J ) * yV( 4 ) -     xB3 * yV( 5 ) ) 
     zV(3) = xV(3) + (      TwoH * yV( 4 ) + B4( J ) * yV( 5 ) )
     zV(4) = xV(4) + (    xB3 * yV(1) + B2(J) * yV(2) - Twox * B4(J) * yV(3) ) 
     zV(5) = xV(5) + ( rho(J) * yV(1) - B1(J) * yV(2) -       FourHx * yV(3) ) 

     ! Scale if necessary
     IF ( II /= N( Medium ) ) THEN 
        IF ( ABS( DBLE( zV( 2 ) ) ) < Floor ) THEN
           zV   = Roof * zV
           yV   = Roof * yV 
           IPow = IPow - IPowR 
        ENDIF

        IF ( ABS( DBLE( zV( 2 ) ) ) > Roof  ) THEN 
           zV   = Floor * zV
           yV   = Floor * yV 
           IPow = IPow - IPowF 
        ENDIF
     ENDIF
  END DO

  yV = ( xV + 2.0 * yV + zV ) / 4.0   ! Apply the standard filter at the terminal point

  RETURN 
END SUBROUTINE ELASDN
