SUBROUTINE EVAL3D( k, PhiR, PhiS, M, IELTSC, xs, ys, theta, Ntheta, RminM, RmaxM, NR, MACT, P )                               

  ! Computes 3-D pressure field using adiabatic mode theory           
  ! Normalized to pressure of point source at 1 meter                 
  ! Note RminM must be zero

  USE ElementMod                                          

  INTEGER ZBARFIL 
  COMPLEX i 
  PARAMETER ( KBARFIL = 55, ZBARFIL = 56, PI = 3.141592, DEGRAD = PI/180.0, i = ( 0.0, 1.0 ), maxM = 200 )
  INTEGER   M( * ), Outside 
  REAL      theta( Ntheta ) 
  COMPLEX   PhiR( maxM, * ), PhiIn( maxM ), PhiOut( maxM ), PhiS( maxM, * ), CONST( maxM ), &
       k(   maxM, * ),  kIn( maxM ),  kOut( maxM ), P( Ntheta, * ), T, kInT, PhiInT
  COMPLEX (KIND=8), ALLOCATABLE :: sumk( : ) 

  ! Open file for eigenfunctions                                  
  OPEN ( FILE = 'ZBARFIL', UNIT = ZBARFIL, STATUS = 'UNKNOWN' ) 

  !  *** Loop over angle ***                                           

  DELTAR = ( RmaxM - RminM ) / ( NR - 1 ) 

  DO Itheta = 1, Ntheta 
     tsx = COS( DEGRAD * theta( Itheta ) )
     tsy = SIN( DEGRAD * theta( Itheta ) ) 

     ! Get modal values                                           
     Ielt = IeltSC 
     WRITE( *, * ) 
     WRITE( *, * ) 'Itheta, tsx, tsy', Itheta, tsx, tsy 

     CALL FIRST( Ielt, Outside, RIn, ROut, XS, YS, tsx, tsy,        &
          MPROP, M, maxM, k, PhiR, PhiS, CONST, kIn, PhiIn, kOut, PhiOut )                          

     MPROP = MIN( MACT, MPROP ) 

     ! write modes at first range                                 
     IF ( Itheta == 1 ) THEN 
        WRITE( ZBARFIL, * ) MPROP 
        WRITE( ZBARFIL, * ) CONST( 1 : MPROP ) 
     END IF

     CONST( 1:MPROP ) = i * SQRT( 2.0 * PI ) * EXP( i * PI / 4.0 ) * CONST( 1:MPROP )

     IF ( Itheta == 1 ) ALLOCATE( sumk( MPROP ) )
     sumk = 0.0

     ! *** March forward in range ***                                 

     DO IR = 1, NR 
        RM = RminM + ( IR - 1 ) * DELTAR 
        IF ( RM == 0.0 ) RM = MIN( 1.0, DELTAR ) 

        ! Crossing into new element?                                  

        DO WHILE ( RM > ROut ) 

           ! Copy outside info to inside                          
           NEWELT = AdjElt( Outside, Ielt ) 
           RIn = ROut 

           kIn(  1:MPROP ) =  kOut( 1:MPROP ) 
           PhiIn( 1:MPROP ) = PhiOut( 1:MPROP ) 

           ! Get new outside info                                 
           CALL OUT( Ielt, NEWELT, Outside, ROut, XS, YS, tsx, tsy, MPROP, M, maxM, k, PhiR, kOut, PhiOut )             
           Ielt = NEWELT 

        END DO

        ! *** Compute modal contribution at this range ***            

        T = 0.0 

        IF ( RIn /= ROut ) THEN 
           ALPHA = ( RM - RIn ) / ( ROut - RIn ) 
           ALPHA = MIN( MAX( ALPHA,  0.0 ), 1.0 ) 
        ELSE 
           ALPHA = 0.0 
        ENDIF

        IF ( IR == NR ) WRITE( ZBARFIL, * ) MPROP 
        DO L = 1, MPROP 
           kInT     =  kIn( L ) + ALPHA * (  kOut(L) -  kIn(L) ) 
           PhiInT    = PhiIn( L ) + ALPHA * ( PhiOut(L) - PhiIn(L) ) 
           sumk( L ) =  sumk( L ) + DELTAR * kInT 
           T         = T + PhiInT * CONST( L ) * EXP( -i * sumk( L ) ) / SQRT( kInT )       
           IF ( IR == NR ) WRITE( ZBARFIL, * ) PhiInT  ! write mode at last range     
        END DO

        P( Itheta, IR ) = T / SQRT( RM ) 

     END DO   ! Next range

     ! Write average wavenumber to file                           
     OPEN ( FILE = 'KBARFIL', UNIT = KBARFIL, STATUS = 'UNKNOWN' ) 
     WRITE( KBARFIL, * ) MPROP, RM 
     WRITE( KBARFIL, * ) sumk( 1:MProp ) / RM

  END DO ! Next bearing

  RETURN 
END SUBROUTINE EVAL3D
!**********************************************************************C
SUBROUTINE FIRST( Ielt, Outside, RIn, ROut, XS, YS, tsx, tsy,     &
     MPROP, M, maxM, k, PhiR, PhiS, CONST, kIn, PhiIn, kOut, PhiOut )

  ! Given an element number for the source                            
  ! Computes                                                          
  !    Outside    the side through which path exits                   
  !    ROUT       the range at which path exits                       
  !    Mode values at entrance and exit                               
  !    Interpolated mode excitation coeffs                            

  USE ElementMod
  INTEGER Outside, M( * ), ICOR( 3, 2 ) 
  REAL    RV( 3 ), SV( 3 ), RVC( 3 ) 
  COMPLEX, INTENT( IN ) :: PhiS( maxM, * ), PhiR( maxM, * ), kIn( * ), kOut( * ), k( maxM, * )
  COMPLEX PhiIn( * ), PhiOut( * ), Const( * )          
  DATA ( (ICOR( IS, J), IS = 1, 3), J = 1, 2) /1, 2, 3, 2, 3, 1/ 

  ! ICOR maps a side (1, 2 OR 3) and a local node (1 OR 2) to a       
  !      corner (1, 2, or 3) of the triangle                        

  WRITE( *, * ) 'Ielt = ', Ielt 
  MPROP = 9999 

  ! Coordinates of the centroid of the source element                 

  XCEN = SUM( X( node( 1:3, Ielt ) ) ) / 3.0   ;   YCEN = SUM( Y( node( 1:3, Ielt ) ) ) / 3.0 

  DO ISIDE = 1, 3 

     node1 = node( ICOR( ISIDE, 1 ), Ielt )   ;   node2 = node( ICOR( ISIDE, 2 ), Ielt )
     Iset1 = Iset( node1 )                    ;   Iset2 = Iset( node2 ) 
     MPROP = MIN( MPROP, M( Iset1 ), M( Iset2 ) ) 

     X1S = X( node1 ) - XS         ;         Y1S = Y( node1 ) - YS
     TXC = X( node1 ) - XCEN       ;         TYC = Y( node1 ) - YCEN                                 
     TX  = X( node2 ) - X( node1 ) ;         TY  = Y( node2 ) - Y( node1 ) 

     DELTA = tsx * TY - tsy * TX 

     ! *** Radial parallel to side? ***                               

     IF ( DELTA == 0.0 ) THEN 
        SV(  ISIDE )    = 999.0 
     ELSE 
        RVC( ISIDE ) = ( TXC * TY  - TYC * TX  ) / DELTA 
        RV(  ISIDE ) = ( X1S * TY  - Y1S * TX  ) / DELTA 
        SV(  ISIDE ) = ( X1S * tsy - Y1S * tsx ) / DELTA 
     ENDIF
  END DO

  ! Identify two good sides and one bad side based on the intercept point

  IBAD = 1 
  IF ( ABS( SV( 2 ) - 0.5 ) > ABS( SV( IBAD ) - 0.5 ) ) IBAD = 2                                                       
  IF ( ABS( SV( 3 ) - 0.5 ) > ABS( SV( IBAD ) - 0.5 ) ) IBAD = 3                                                       

  IGOOD1 = 1
  IGOOD2 = 2
  IF ( IBAD == 1 ) THEN 
     IGOOD1 = 3 
  ELSE IF ( IBAD == 2 ) THEN 
     IGOOD2 = 3 
  ENDIF

  ! The side with the lesser RVC is the inside                    

  IF ( RVC( IGOOD1 ) < RVC( IGOOD2 ) ) THEN 
     Inside  = IGOOD1   ;   Outside = IGOOD2 
  ELSE 
     Inside  = IGOOD2   ;   Outside = IGOOD1 
  ENDIF

  SIn      = SV( Inside ) 
  SIn      = MIN( MAX( SIn,  0.0 ), 1.0 ) 
  RIn      = RV( Inside ) 

  SOut     = SV( Outside ) 
  SOut     = MIN( MAX( SOut, 0.0 ), 1.0 ) 
  ROut     = RV( Outside ) 

  ! Get values of modes at Z = SD and (X, Y) = intercept points   

  ICOR1 = ICOR(  Inside, 1 )   ;   ICOR2 = ICOR(  Inside, 2 ) 
  ICOR3 = ICOR( Outside, 1 )   ;   ICOR4 = ICOR( Outside, 2 ) 

  ! Interpolate to get modal values at source                     
  R = 0.0 

  IF ( RIn /= ROut ) THEN 
     ALPHA = ( R - RIn ) / ( ROut - RIn ) 
     ALPHA = MIN( MAX( ALPHA,  0.0 ), 1.0 ) 
  ELSE 
     ALPHA = 0.0 
  ENDIF

  DO L = 1, MPROP 
     PhiIn(  L ) = PhiS( L, ICOR1 ) + SIn  * ( PhiS( L, ICOR2 ) - PhiS( L, ICOR1 ) )   
     PhiOUT( L ) = PhiS( L, ICOR3 ) + SOut * ( PhiS( L, ICOR4 ) - PhiS( L, ICOR3 ) )   
     CONST(  L ) = PhiIn(L) + ALPHA * ( PhiOut(L) - PhiIn(L) ) 
  END DO

  ! Obtain values of modes at z = Rd and (x, y)=intercept points      

  CALL INTER1( Ielt, Inside,  SIn,  MPROP, M, maxM, k, PhiR, kIn,  PhiIn  )          
  CALL INTER1( Ielt, Outside, SOut, MPROP, M, maxM, k, PhiR, kOut, PhiOut )          

  RETURN 
END SUBROUTINE FIRST
!**********************************************************************C
SUBROUTINE OUT( Ielt, NEWELT, Outside, ROut, XS, YS,&
     &   tsx, tsy, MPROP, M, maxM, k, PhiR, kOut, PhiOut)          

  ! Given an element number and a side through which prop path enters 
  ! Computes                                                          
  !    Outside    the side through which path exits                   
  !    ROut       the range at which path exits                       
  !    Mode values                                                    

  USE ElementMod
  INTEGER Outside, M( * ), ICOR( 3, 2 )
  COMPLEX PhiOut( * ), PhiR( maxM, * ), kOut( * ), k( maxM, * ) 
  DATA ( (ICOR( IS, J), IS = 1, 3), J = 1, 2) /1, 2, 3, 2, 3, 1/ 

  ! ICOR maps a side (1, 2 OR 3) and a local node (1 OR 2) to a       
  !        corner (1, 2, or 3) of the triangle                        
  ! WRITE( *, * ) '   Crossing into new element = ', NEWELT           

  ! If no adj elt then exit. Previous values of KOut, PhiOut retained 

  IF ( NEWELT == 0 ) THEN 
     ROut = 9.9999E9 
     RETURN 
  ENDIF

  ! *** Loop over sides to find outside ***                           

  SOut = 999.0 

  DO ISIDE = 1, 3 

     ! Don't try an outside the same as the inside             
     IF ( AdjElt( ISIDE, NEWELT ) /= Ielt ) THEN 

        node1 = node( ICOR( ISIDE, 1 ), NEWELT ) 
        node2 = node( ICOR( ISIDE, 2 ), NEWELT ) 

        X1S = X( node1 ) - XS           ;   Y1S = Y( node1 ) - YS                                               
        TX  = X( node2 ) - X( node1 )   ;   TY  = Y( node2 ) - Y( node1 ) 

        DELTA = tsx * TY - tsy * TX 

        ! *** Radial parallel to side? ***                            

        IF ( DELTA == 0.0 ) THEN 
           ST    = 999.0 
        ELSE 
           ROutT = ( X1S * TY  - Y1S * TX  ) / DELTA 
           ST    = ( X1S * tsy - Y1S * tsx ) / DELTA 
        ENDIF
        ! If intercept is in segment, store the side number       
        IF ( ABS( ST - 0.5 ) < ABS( SOut - 0.5) ) THEN 
           Outside = ISIDE 
           SOut    = ST 
           ROut    = ROutT 
        ENDIF
     ENDIF
  END DO

  ! Obtain values of modes at intercept points                        

  CALL INTER1( NEWELT, Outside, SOut, MPROP, M, maxM, k, PhiR, kOut, PhiOut )          

  RETURN 
END SUBROUTINE OUT
!**********************************************************************C
SUBROUTINE INTER1( Ielt, ISIDE, S, MPROP, M, maxM, k, PhiR, kInT, PhiInT )          

  ! Given                                                             
  !    Ielt         the element                                       
  !    ISIDE        the side                                          
  !    S            the proportional distance along the side          

  ! Returns                                                           
  !    kInT, PhiInT  interpolated modal values                       
  !    MPROP          number of propagating modes                     

  USE ElementMod
  INTEGER M( * ), ICOR( 3, 2 ) 
  COMPLEX PhiInT( * ), PhiR( maxM, * ), kInT( * ), k( maxM, * )
  DATA ( (ICOR( IS, J), IS = 1, 3), J = 1, 2) /1, 2, 3, 2, 3, 1/ 

  ! ICOR maps a side (1, 2 OR 3) and a local node (1 OR 2) to a       
  !        corner (1, 2, or 3) of the triangle                        

  node1 = node( ICOR( ISIDE, 1 ), Ielt )   ;   node2 = node( ICOR( ISIDE, 2 ), Ielt )                                         
  Iset1 = Iset( node1 )                    ;   Iset2 = Iset( node2 ) 
  MPROP = MIN( MPROP, M( Iset1 ), M( Iset2 ) ) 

  ! Extrapolation is blocked by making sure s is in [0, 1]                                   

  S = MIN( MAX( S, 0.0 ), 1.0 ) 

  DO I = 1, MPROP 
     kInT(   I ) =    k( I, Iset1 ) + S * (    k( I, Iset2 ) -    k( I, Iset1 ) )  
     PhiInT( I ) = PhiR( I, Iset1 ) + S * ( PhiR( I, Iset2 ) - PhiR( I, Iset1 ) )  
  END DO

  RETURN 
END SUBROUTINE INTER1
