PROGRAM FIELD

  ! Generate file of replica vectors

  USE SdRdRMod

  IMPLICIT NONE
  INTEGER, PARAMETER :: FLPFil = 5, PrtFil = 6, ShdFil = 25, MaxM = 10000  ! MaxM also occurs in Eval, EvalAD, EvalCM
  INTEGER   I, IR, iProf, NProf, Nrr, IAllocStat, iRec, IS, M, MLimit, MSrc
  REAL      zMin, zMax, freq, atten
  COMPLEX   k( MaxM )
  CHARACTER Opt*50, SHDTitle*80, Title*80, Comp*1, PlotType*10, FileName*6
  REAL,    ALLOCATABLE :: rr( : ), rProf( : )
  COMPLEX, ALLOCATABLE :: phiS( :, : ), phiR( :, : ), P( :, : ), C( : ), Ptemp( : )

  WRITE( PRTFIL, * ) '__________________________________________________'
  WRITE( PRTFIL, * ) 'Running FIELD'

  SHDTitle(1:1) = '$'

  READ( FLPFil, * ) SHDTitle
  READ( FLPFil, * ) Opt
  READ( FLPFil, * ) MLimit

  Comp   = Opt( 3:3 )
  MLimit = MIN( MaxM, MLimit )

  ! *** Read profile ranges ***

  READ( FLPFil, * ) NProf
  ALLOCATE( rProf( MAX( 3, NProf + 1 ) ), Stat = IAllocStat )   ! NProf + 1 profiles (one added at `infinite' range)

  IF ( IAllocStat /= 0 ) THEN
     WRITE( PrtFil, * ) 'NProf = ', NProf
     CALL ERROUT( PrtFil, 'F', 'FIELD', 'Dynamic memory allocation failed: Too many profiles' )
  ENDIF

  IF ( NProf > 2 ) rProf( 3 ) = -999.9
  READ( FLPFil, * ) rProf( 1 : NProf )
  CALL SUBTAB( rProf, NProf )

  WRITE( PRTFIL, * )
  WRITE( PRTFIL, * ) 'Number of profiles   = ', NProf
  IF ( NProf >= 1  ) WRITE( PRTFIL, "( 5G14.6 )" ) ( rProf( iProf ), iProf = 1, MIN( NProf, 51 ) )
  IF ( NProf > 51  ) WRITE( PRTFIL, * ) ' ... ', rProf( NProf )

  ! EVALAD/EVALCM need a profile at zero range
  IF ( rProf( 1 ) /= 0.0 ) CALL ERROUT( PrtFil, 'F', 'FIELD', 'The first profile must be at 0 km' )

  CALL RANGES( FLPFil, PrtFil )           ! Read receiver ranges
  zMin = -1.0E10
  zMax = +1.0E10
  CALL SDRD( FLPFil, PrtFil, zMin, zMax ) ! Read source/receiver depths

  ALLOCATE( phiS( MaxM, Nsd ), phiR( MaxM, Nrd ), C( MaxM ), Stat = IAllocStat )
  IF ( IAllocStat /= 0 ) THEN
     WRITE( PrtFil, * ) 'Nrd = ', Nrd
     CALL ERROUT( PrtFil, 'F', 'FIELD', 'Dynamic memory allocation failed: Too many receiver depths' )
  ENDIF

  ! *** Read receiver ranges (offsets from vertical) ***

  READ( FLPFil, * ) Nrr

  IF ( Nrr /= Nrd ) THEN
     WRITE( PrtFil, * ) 'Nrr, Nrd = ', Nrr, Nrd
     CALL ERROUT( PrtFil, 'W', 'FIELD', 'Nrr being set to Nrd' )
     Nrr = Nrd
  ENDIF

  ALLOCATE( rr( Nrr ) )
  IF ( Nrr > 1 ) rr( 2 ) = -999.9
  IF ( Nrr > 2 ) rr( 3 ) = -999.9
  READ( FLPFil, * ) rr( 1 : Nrr )

  CALL SUBTAB( rr, Nrd )
  WRITE( PrtFil, * )
  WRITE( PrtFil, * ) 'Number of receiver range offsets = ', Nrd
  WRITE( PRTFIL, * ) 'Receiver range offsets (km)'

  IF ( Nrd >= 1 ) WRITE( PrtFil,  "( 5G14.6 )" ) ( rr( IR ), IR = 1, MIN( Nrd, 51 ) )
  IF ( Nrd > 51 ) WRITE( PRTFIL, * ) ' ... ', rr( Nrd )

  WRITE( PrtFil, * )

  ALLOCATE ( P( Nrd, Nr ), Ptemp( Nr ), Stat = IAllocStat )
  IF ( IAllocStat /= 0 ) STOP "Fatal Error: Insufficient memory to allocate P( Nrd, Nr )"

  !  *** Read in modes ***

  IPROF = 1
  CALL GETMOD( IPROF, ' ', MaxM, sd, Nsd, 'N' , k, phiS, MSrc, freq, Title )
  CALL GETMOD( IPROF, ' ', MaxM, rd, Nrd, Comp, k, phiR, MSrc, freq, Title )

  ! Generate header
  FileName = 'SHDFIL'
  IF ( SHDTitle(1:1) == '$' ) SHDTitle = Title
  PlotType   = '          '
  atten      = 0.0
  ALLOCATE( theta( 1 ) )
  theta( 1 ) = 0   ! dummy bearing angle
  Ntheta     = 1
  CALL WriteHeader( FileName, SHDTitle, theta, Ntheta, sd, Nsd, rd, Nrd, R, Nr, freq, atten, PlotType, 0.0, 0.0 )
  iRec = 7

  ! *** MAIN LOOP: For each source evaluate and write the field ***

  DO IS = 1, Nsd
     M = MIN( MLimit, MSrc )   ! Set number of propagating modes

     C( 1 : MSrc ) = phiS( 1 : MSrc, is )
     IF    ( NProf == 1      ) THEN   ! Range-independent case
           CALL EVAL(                 C, phiR,     Nrd, R, Nr, rr, k, M, Opt, P )
     ELSE
        IF ( Opt(2:2) == 'C' ) THEN   ! Range-  dependent case
           ! Coupled mode theory
           CALL EVALCM( rProf, NProf, C, phiR, rd, Nrd, R, Nr,     k, M, Opt, P )
        ELSE
           ! Adiabatic mode theory
           CALL EVALAD( rProf, NProf, C, phiR, rd, Nrd, R, Nr,        M, Opt, P )
        ENDIF
     ENDIF

     ! write out the field
     DO I = 1, Nrd
        IRec = IRec + 1
        Ptemp = P( i, 1 : Nr )   ! temporary variable, avoids compiler warning
        WRITE( SHDFil, REC = IRec ) Ptemp
     END DO

  END DO   ! next source depth
  CLOSE( ShdFil )

  STOP
END PROGRAM FIELD
