SUBROUTINE ZSECCX( x2, Tolerance, Iteration, MAXIteration, ErrorMessage )

  ! Secant method

  IMPLICIT NONE
  INTEGER            :: IPower0, IPower1, Iteration, MAXIteration
  REAL    ( KIND=8 ) :: Tolerance
  COMPLEX ( KIND=8 ) :: x0, x1, x2, shift, f0, f1, CNUM, CDEN
  CHARACTER*80          ErrorMessage 

  ErrorMessage = ' ' 
  IF ( Tolerance <= 0.0 ) THEN 
     ErrorMessage = 'Non-positive tolerance specified' 
     STOP 
  ENDIF

  x1     = x2 + 100.0 * Tolerance 
  CALL FUNCT( x1, f1, IPower1 ) 

  !WRITE( *, * )
  !WRITE( *, "( 4G17.9, I5 )" ) SQRT( x1 ), f1, IPower1

  DO Iteration = 1, MAXIteration 
     x0 = x1
     f0 = f1 
     IPower0 = IPower1 

     x1 = x2 
     CALL FUNCT( x1, f1, IPower1 )

     ! ugly stuff to block overflows by forcing shift to be bounded
     CNUM = f1 * ( x1 - x0 )

     ! gfortran got a perfect 0, leading to NaN for the shift
     ! following traps that
     ! the number 30 roughly follows from the Floor/Roof thresholds
     IF ( IPower0 - IPower1 > 30 ) RETURN

     CDEN = f1 - f0 * 10.0 ** ( IPower0 - IPower1 )

     IF ( ABS( CNUM ) >= ABS( CDEN * x1 ) ) THEN 
        shift = 100.0 * Tolerance * ABS( x1 ) 
     ELSE 
        shift = CNUM / CDEN 
     ENDIF

     x2 = x1 - shift 
     !WRITE( *, "( 4G17.9, I5 )" ) SQRT( x1 ), f1, IPower1

     IF ( ABS( x2 - x1 ) + ABS( x2 - x0 ) < Tolerance ) RETURN                       
  ENDDO

  ErrorMessage = ' *** FAILURE TO CONVERGE IN SECANT' 

  RETURN 
END SUBROUTINE ZSECCX
