function h = reducestep( SSP, x0, Tray, Layer0, c, xTop, nTop, xBot, nBot, rTopSeg, rBotSeg, deltas, h )

x = x0 + h * c * Tray;  % make a trial step

% *** Detect interface or boundary crossing

% *** Adjust step to land on an interface or boundary

hLayer = inf;
if ( x( 2 ) < SSP.z( Layer0 ) )
   hLayer = ( SSP.z( Layer0 ) - x0( 2 ) ) / ( Tray( 2 ) * c );
end

Layer1 = Layer0 + 1;

if ( x( 2 ) > SSP.z( Layer1 ) )
   hLayer = ( SSP.z( Layer1 ) - x0( 2 ) ) / ( Tray( 2 ) * c );
end

% top crossing
hTop = inf;
d    = x - xTop;       % vector from top    to ray endpoint

if ( d * nTop' > 0 )   % top crossing
   e =  x0 - xTop;     % vector from top    to ray origin
   hTop = -e * nTop' / ( c * Tray * nTop' );
end

% bottom crossing
hBot = inf;
d    = x - xBot;       % vector from bottom to ray endpoint

if ( d * nBot' > 0 )   % bottom crossing
   e =  x0 - xBot;     % vector from bottom to ray origin
   hBot = -e * nBot' / ( c * Tray * nBot' );
end

% top segment crossing in range
h4 = inf;

if ( Tray( 1 ) ~= 0 )
   if    ( x( 1 ) < rTopSeg( 1 ) )
         h4 = -( x0( 1 ) - rTopSeg( 1 ) ) / ( Tray( 1 ) * c );
   else
      if ( x( 1 ) > rTopSeg( 2 ) )
         h4 = -( x0( 1 ) - rTopSeg( 2 ) ) / ( Tray( 1 ) * c );
      end
   end
end

% bottom segment crossing in range
h5 = inf;

if ( Tray( 1 ) ~= 0 )
   if    ( x( 1 ) < rBotSeg( 1 ) )
         h5 = -( x0( 1 ) - rBotSeg( 1 ) ) / ( Tray( 1 ) * c );
   else
      if ( x( 1 ) > rBotSeg( 2 ) )
         h5 = -( x0( 1 ) - rBotSeg( 2 ) ) / ( Tray( 1 ) * c );
      end
   end
end

h = min( [ h, hLayer, hTop, hBot, h4, h5 ] );   % step is limited by the smaller of an interface or bottom crossing
h = max( h, 1e4 * eps( deltas ) );   % make sure we make some motion
