function [ Asparse, Bsparse ] = setAB(  B1, B2, B3, B4, rho, Mater )

% Initializes arrays defining difference equations

global NMedia N cInsT rhoInsT cInsB rhoInsB
global omega Bdry
global Loc H

X=0
omega2 = omega^2;

% Compute size of matrix
NMat = 0;
for MED = 1 : NMedia
    switch Mater(MED, : )
        case 'ACOUSTIC'
            NMat = NMat + N( MED ) + 1;
        case 'ELASTIC '
            N( MED )
            NMat = NMat + 4 * ( N( MED ) + 1 );
    end
end

A = zeros( 13, NMat ); % Zero out the matrix
B = zeros( 13, NMat ); % Zero out the matrix

% MAIN LOOP ( OVER EACH MEDIUM )

J = 0;
for MED = 1 : NMedia
    if ( MED ~= 1 )

        % Acousto-elastic interface
        if ( strcmp( Mater(MED - 1, : ), 'ACOUSTIC' ) && strcmp( Mater(MED,: ), 'ELASTIC ' ) )
            % R2 Condition
            L = L + 1;
            J = J + 1;
            A( 10, J - 1 ) = 1.0;
            A(  9, J     ) = 0.5 * B1(L);
            B(  9, J     ) = 0.5 * H( MED - 1 )^2;
            A(  7, J + 2 ) = H(MED - 1) * omega2;
            % R3 Condition
            A( 10, J     ) = 0.0;
            A(  7, J + 3 ) = 1.0;
            % R4 Condition
            A( 11, J     ) = 1.0;
            A(  7, J + 4 ) = 1.0;
        end

        % Elasto-acoustic interface

        if ( strcmp( Mater(MED - 1,: ), 'ELASTIC ' ) && strcmp( Mater(MED,: ), 'ACOUSTIC' ) )
            L = Loc(MED) + 1;
            J = J + 5;
            % R3 Condition
            A(9, J - 2) = 1.0;
            % R4 Condition
            A(9, J - 1) = 1.0;
            A(8, J  ) = 1.0;
            % R2 Condition
            A(12, J - 3 ) = -H(MED) * omega2;
            A( 9, J     ) = 0.5 * B1(L);
            B( 9, J     ) = 0.5 * H(MED)^2;
            A( 8, J + 1 ) = 1.0;
        end

        % Acoustic/acoustic interface

        if ( strcmp( Mater(MED - 1,: ), 'ACOUSTIC' ) && strcmp( Mater(MED,: ), 'ACOUSTIC' ) )
            % Continuity of normal displacement
            L = L + 1;
            J = J + 1;
            Hrho = H(MED - 1) * rho( Loc(MED - 1) + 1 );
            A( 10, J - 1 ) = 1.0 / Hrho;
            A(  9, J   ) = 0.5 * B1(L) / Hrho;
            B(  9, J   ) = 0.5 * H(MED - 1)^2 / Hrho;
            L = L + 1;
            Hrho = H(MED) * rho( Loc(MED) + 1 );
            A(  8, J + 1 ) = 0.5 * B1(L) / Hrho;
            B(  8, J + 1 ) = 0.5 * H(MED)^2 / Hrho;
            A(  7, J + 2 ) = 1.0 / Hrho;
            % Continuity of pressure
            J = J + 1;
            A( 10, J - 1 ) = 1.0;
            A(  9, J   ) = -1.0;
        end

        % Elastic/elastic interface

        if ( strcmp( Mater(MED - 1,: ), 'ELASTIC ' ) && strcmp( Mater(MED,: ), 'ELASTIC ' ) )
            % Continuity of everything
            J = J + 1;
            A( 13, J - 2 ) =  1.0;
            A(  9, J + 2 ) = -1.0;
            J = J + 1;
            A( 13, J - 2 ) =  1.0;
            A(  9, J + 2 ) = -1.0;
            J = J + 1;
            A( 13, J - 2 ) =  1.0;
            A(  9, J + 2 ) = -1.0;
            J = J + 1;
            A( 13, J - 2 ) =  1.0;
            A(  9, J + 2 ) = -1.0;
        end
    end

    % this section for the volume
    
    switch Mater(MED,: )
        case 'ACOUSTIC' % Acoustic section
            L = Loc(MED) + 1;
            if ( MED == 1 )
                J = 1;
            end

            for II = 1 : N(MED) - 1
                J = J + 1;
                L = L + 1;
                A( 10, J - 1 ) = 1.0;
                A(  9, J   ) = B1(L);
                B(  9, J   ) = H(MED) * H(MED);
                A(  8, J + 1 ) = 1.0;
            end
        case 'ELASTIC '  % Elastic section

            L = Loc(MED);
            TWOBYH = 2.0 / H(MED);
            for II = 1 : N(MED)
                L = L + 1;
                J = J + 1;
                A( 11, J ) = TWOBYH;
                B( 12, J ) = -B4(L);
                A( 13, J ) = -rho(L);
                B( 13, J ) = -B3(L);
                J = J + 1;
                A( 10, J ) = -1.0;
                A( 11, J ) = TWOBYH;
                A( 13, J ) = -rho(L);
                J = J + 1;
                A(  9, J ) = B1(L);
                A( 11, J ) = TWOBYH;
                B( 12, J ) = -1;
                J = J + 1;
                A(  9, J ) = B2(L);
                A( 10, J ) = -B4(L);
                A( 11, J ) = TWOBYH;

                L = L + 1;
                J = J + 1;
                A( 7, J ) = -TWOBYH;
                B( 8, J ) = -B4(L);
                A( 9, J ) = -rho(L);
                B( 9, J ) = -B3(L);
                J = J + 1;
                A( 6, J ) = -1.0;
                A( 7, J ) = -TWOBYH;
                A( 9, J ) = -rho(L);
                J = J + 1;
                A( 5, J ) = B1(L);
                A( 7, J ) = -TWOBYH;
                B( 8, J ) = -1;
                J = J + 1;
                A( 5, J ) = B2(L);
                A( 6, J ) = -B4(L);
                A( 7, J ) = -TWOBYH;
                J = J - 4;
                L = L - 1;
            end
    end
end   % next medium

% Top BC

switch Mater(1,: )
    case 'ELASTIC '  % Elastic medium
        switch Bdry.Top.Opt(2:2)
            case 'R'  % Rigid top
                A( 9, 1 ) = 1.0;
                A( 9, 2 ) = 1.0;
            case 'V'  % Free (vacuum) top
                A( 7, 3 ) = 1.0;
                A( 7, 4 ) = 1.0;
        end
    case 'ACOUSTIC' % Acoustic medium
        % Top impedance
        BCType(1:1) = Bdry.Top.Opt(2:2);
        [ F, G, IPow ] = bcimp( B1, B2, B3, B4, rho, X, BCType, 'TOP', Bdry.Top.cp, Bdry.Top.cs, Bdry.Top.rho, cInsT, rhoInsT );
        if ( G == 0.0 )  % Free (vacuum) top
            A( 9, 1 ) = 1.0;
            A( 8, 2 ) = 0.0;
        else
            Hrho = rho(1) * H(1);
            A( 9, 1 ) = 0.5 * B1(1) / Hrho + F / G;
            B( 9, 1 ) = 0.5 * H(1)^2 / Hrho - F / G;
            A( 8, 2) = 1.0 / Hrho;
        end
end

% Bottom BC

switch Mater(NMedia,: )
    case 'ELASTIC '  % Elastic medium
        switch Bdry.Bot.Opt(1:1 )
            case 'R'  % Rigid bottom
                A( 11, NMat - 3 ) = 1.0;
                A( 11, NMat - 2 ) = 1.0;
            case 'V'  % Free (vacuum) bottom
                A( 9, NMat - 1 ) = 1.0;
                A( 9, NMat   ) = 1.0;

            case 'A'  % Elastic bottom
                error( 'need to fix eigenvalue in this section' )
%                 GAMP2 = X - omega2 / CPB^2;
%                 GAMS2 = X - omega2 / CSB^2;
%                 GAMP = SQRT( GAMP2 );
%                 GAMS = SQRT( GAMS2 );
%                 RMU = rhoB * CSB^2;
%                 FAC = -RMU / ( GAMS * GAMP - X );
% 
%                 A( 11, NMat - 3 ) = FAC * GAMP * (X - GAMS2);
%                 A( 10, NMat - 2 ) = FAC * (2.0 * GAMS * GAMP - (GAMS2 + X));
%                 A( 12, NMat - 3 ) = X * A(10, NMat - 2);
%                 A( 11, NMat - 2 ) = FAC * GAMS * (X - GAMS2);
%                 A(  9, NMat - 1 ) = 1.0;
%                 A(  9, NMat   ) = 1.0;
        end
    case 'ACOUSTIC' % Acoustic medium
        % Bottom impedance
        BCType(1:1) = Bdry.Bot.Opt(1:1);
        [ F, G, IPow ] = bcimp( B1, B2, B3, B4, rho, X, BCType, 'BOT', Bdry.Bot.cp, Bdry.Bot.cs, Bdry.Bot.rho, cInsB, rhoInsB );
        
        if ( G == 0.0 )  % Free (vacuum) bottom
            A( 10, NMat - 1 ) = 0.0;
            A(  9, NMat   ) = 1.0;
        else
            L = L + 1;
            Hrho = H(NMedia) * rho( Loc(NMedia) + 1 );
            A( 10, NMat - 1) = 1.0 / Hrho;
            A(  9, NMat  ) = 0.5  *  B1( L ) / Hrho - F / G;
            B(  9, NMat  ) = 0.5  *  H( NMedia )^2 / Hrho - F / G;
        end
end

Asparse = spdiags( A( 5:13, : ).', 4:-1:-4, NMat, NMat );
Bsparse = spdiags( B( 5:13, : ).', 4:-1:-4, NMat, NMat );
