function field( filename, modes )

% calculates the field using modes produced by KRAKEN
% parameters read from field.flp
%
% usage: field( filename, modes )
% mbp

%Comp     = 'P';   % select component to plot (P, H, V) (Pressure, Horizontal, Vertical)

[ TitleEnv, Opt, Comp, MLimit, NProf, rProf, R, Pos ] = read_flp;

if nargin == 1
  [ PlotTitle, freq, Modes ] = read_modes( filename );
else
  [ PlotTitle, freq, Modes ] = read_modes( filename, modes );
end
MSrc = length( Modes.k );

% weight modes by mode excitation
zs  = Pos.s.depth;
isd = find( Modes.z >= zs );		% index of source depth
isd = isd( 1 );
C   = Modes.phi( isd, : ).';   % column vector with modal weights

% calculate mode values at receiver depths
irdvec = zeros( 1, length( Pos.r.depth ) );
for ii = 1 : length( Pos.r.depth )
   zr  = Pos.r.depth( ii );
   ird = find( Modes.z >= zr );		% index of source depth
   irdvec( ii ) = ird( 1 );
end
phiR = Modes.phi( irdvec, : );

rd  = Pos.r.depth;
Nrd = length( rd );
Nr  = length( R );

M = min( MLimit, MSrc );   % Set number of propagating modes

if ( NProf == 1 )   % Range-independent case
   p = evalri( C, phiR, R, Modes.k, Opt, Comp );
else
   if ( Opt(2:2) == 'C' )   % Range-  dependent case
      % Coupled mode theory
      p = evalcm( rProf, NProf, C, phiR, rd, Nrd, R, Nr, Modes.k, M, Opt );
   else
      % Adiabatic mode theory
      p = evalad( rProf, NProf, C, phiR, rd, Nrd, R, Nr,          M, Opt );
   end
end

PlotType    = 'rectilin  ';
atten       = 0.0;
% Pos.s.depth = zs;
% Pos.r.depth = Modes.z;
Pos.r.range = R;

save SHDFIL PlotTitle PlotType freq atten Pos p
