function fieldLoadAll

% calculates the field using modes produced by KRAKEN
% parameters read from field.flp
%
% usage: field( filename, modes )
% mbp

%Comp     = 'P';   % select component to plot (P, H, V) (Pressure,
%Horizontal, Vertical)

filename = 'MODFIL0001';

[ TitleEnv, Opt, Comp, MLimit, NProf, rProf, R, Pos ] = read_flp;
modes = 1 : MLimit;

if nargin == 1
  Modes = read_modes( filename );
else
  Modes = read_modes( filename, modes );
end
MSrc = length( Modes.k );

% optionally read bottom bathymetry
%if ( Bdry.Bot.Opt(2:2) == '*' )
   btyfil = 'DickinsK.bty';
   BotBTY = '*';
   depthB = 3000; % SSP.depth( end );
   rBox   = 30; % RMax;
   readbty( btyfil, BotBTY, depthB, rBox )
%end

%D = interp1( xBot( 1, : ), xBot( 2, : ), r );

% weight modes by mode excitation
zs  = Pos.s.depth;
isd = find( Modes.z >= zs );		% index of source depth
isd = isd( 1 );

C   = Modes.phi( isd, 1 : min( MLimit, Modes.M ) ).';   % column vector with modal weights

% calculate mode values at receiver depths
irdvec = zeros( 1, length( Pos.r.depth ) );
for ii = 1 : length( Pos.r.depth )
   zr  = Pos.r.depth( ii );
   ird = find( Modes.z >= zr );		% index of source depth
   irdvec( ii ) = ird( 1 );
end
phiR = Modes.phi( irdvec, : );

rd  = Pos.r.depth;
Nrd = length( rd );
Nr  = length( R );

M = min( MLimit, MSrc );   % Set number of propagating modes

if ( NProf == 1 )   % Range-independent case
   p = evalri( C, phiR( :, 1 : min( MLimit, Modes.M ) ), R, Modes.k( 1 : min( MLimit, Modes.M ) ), Opt, Comp );
else
   if ( Opt(2:2) == 'C' )   % Range-  dependent case
      % Coupled mode theory
      p = evalcmLoadAll( C, rd, Nrd, R, Nr, M, Opt );
   else
      % Adiabatic mode theory
      p = evalad( rProf, NProf, C, phiR, rd, Nrd, R, Nr,          M, Opt );
   end
end

PlotTitle = Modes( 1 ).title;
PlotType    = 'rectilin  ';
freq        = Modes( 1 ).freq;
atten       = 0.0;
Pos.r.range = R;

save SHDFIL PlotTitle PlotType freq atten Pos p
