
function [ Bathy ] = LoadBathymetry( xyzfile )
% Loads the bathymetry file
% assumed to be a rectangular grid of data written in the standard GEODAS
% xyz form
% mike porter 2/2009 based on earlier version by D. Hernandez

Bathy.fname = xyzfile;
xyz = load( xyzfile );

Lat  = intersect( xyz( :, 2 ), xyz( :, 2 ) ); % get unique latitudes
Long = intersect( xyz( :, 1 ), xyz( :, 1 ) ); % get unique longitudes
Lat  = fliplr( Lat );

nlong   = length( Long );
nlat    = length( Lat  );

z = reshape( xyz( :, 3 ), nlong, nlat );
z = z';
z = flipud( z );

z( z > 0 ) = NaN;   % remove land

Bathy.z    = z;
Bathy.Lat  = Lat;
Bathy.Long = Long;

% Create grid in km
[ LatTotkm,  A12, A21 ] = dist_wh( [Lat( 1 ) Lat( end ) ] , [ Long( 1 ) Long( 1   ) ] );
[ LongTotkm, A12, A21 ] = dist_wh( [Lat( 1 ) Lat( 1   ) ] , [ Long( 1 ) Long( end ) ] );

% mapping of lat/long to km
Bathy.Lat2km  = linspace( 0, LatTotkm,  length( Lat  ) ) / 1000;
Bathy.Long2km = linspace( 0, LongTotkm, length( Long ) ) / 1000;

end



