function plotati( atifil )

% Plot the altimetry file
% usage: plotati( atifil )
% where atifil is the Altimetry file (the extension is optional)
% e.g. plotati( 'foofoo' )
%
% plots the AlTImetrY file used by Bellhop
% MBP July 1999

global xTop NatiPts

if ( strcmp( atifil, 'ATIFIL' ) == 0 && isempty( findstr( atifil, '.ati' ) ) )
  atifil = [ atifil '.ati' ]; % append extension
end

TopATI = '*';   % flag to read from bty file
depthT = -inf;
rBox   = inf;
readati( atifil, TopATI, depthT, rBox ) % read the bathymetry data

% copy, removing +/- inf values
NatiPts = NatiPts - 2;
r = xTop( 1, 2 : end - 1 );
z = xTop( 2, 2 : end - 1 );

% set labels in m or km
xlab     = 'Range (m)';
%if ( max( r ) >= 10000 )
%   r      = r / 1000.0;
%   xlab   = 'Range (km)';
%end
%%

hold on
plot( r, z, 'k-' )
xlabel( xlab )
ylabel( 'Depth (m)' )

zmin = min( z );
zmax = max( z );

r( NatiPts + 1 ) = r( NatiPts );
z( NatiPts + 1 ) = zmin - 0.1 * ( zmax - zmin );
r( NatiPts + 2 ) = r( 1 );
z( NatiPts + 2 ) = zmin - 0.1 * ( zmax - zmin );

fill( r, z, [ 0.7 0.7 1.0 ] );
plot( r, z )

% rmin = min( r );
% rmax = max( r );
% zmin = min( z );
% zmax = max( z );

%axis( [ rmin, rmax, zmin, zmax ] );
set( gca, 'YDir', 'Reverse' )   % because view messes up the zoom feature
 