function plotbty( btyfil )

% Plot the bathymetry file
% usage: plotbty( btyfil )
% where btyfil is the BaThYmetry file (the extension is optional)
% e.g. plotbty( 'foofoo' )
%
% plots the BaThymetrY file used by Bellhop
% MBP July 1999

global xBot NbtyPts

if ( strcmp( btyfil, 'BTYFIL' ) == 0 && isempty( findstr( btyfil, '.bty' ) ) )
  btyfil = [ btyfil '.bty' ]; % append extension
end

BotBTY = '*';   % flag to read from bty file
depthB = inf;
rBox   = inf;
readbty( btyfil, BotBTY, depthB, rBox ) % read the bathymetry data

% copy, removing +/- inf values
NbtyPts = NbtyPts - 2;
r = xBot( 1, 2 : end - 1 );
z = xBot( 2, 2 : end - 1 );

% set labels in m or km
xlab     = 'Range (m)';
%if ( max( r ) >= 10000 )
%   r      = r / 1000.0;
%   xlab   = 'Range (km)';
%end
%%

hold on
plot( r, z, 'k-' )
xlabel( xlab )
ylabel( 'Depth (m)' )

zmin = min( z );
zmax = max( z );

r( NbtyPts + 1 ) = r( NbtyPts );
z( NbtyPts + 1 ) = zmax + 0.1 * ( zmax - zmin );
r( NbtyPts + 2 ) = r( 1 );
z( NbtyPts + 2 ) = zmax + 0.1 * ( zmax - zmin );

fill( r, z, [ 0.5 0.3 0.1 ] );
plot( r, z )

% rmin = min( r );
% rmax = max( r );
% zmin = min( z );
% zmax = max( z );

%axis( [ rmin, rmax, zmin, zmax ] );
set( gca, 'YDir', 'Reverse' )   % plot with depth-axis positive down
