function plotmode( filename, modes )

% plot the modes produced by KRAKEN
%
% usage: plotmode( filename, modes )
% modes is a vector of indices
% can be specified in any order and with duplicates
% mbp

if nargin == 1
   [ Modes ] = read_modes( filename );
   modes = 1 : length( Modes.k );
else
   [ Modes ] = read_modes( filename, modes );
end

%figure; plot( z, psi );
%xlabel( 'Depth (m)' )
%view( 90, 90 )	% rotate

% plots of the wavenumbers in the complex plane

plot( real( Modes.k ), imag( Modes.k ), '.' );
title( Modes.title )
xlabel( 'real( k )' );
ylabel( 'imag( k )' );

% plots of the modes as a single color image
x = 1:size( Modes.phi, 2 );

figure; imagesc( x, Modes.z, real( Modes.phi ) ); colorbar
xlabel( 'Mode index' )
ylabel( 'Depth (m)' )
title( deblank( Modes.title ) )


%%
% line plots of the modes

Nplots = min( length( modes ), 10 )
iskip  = floor( length( modes ) / Nplots );

figure;

for iplot = 1 : Nplots
   subplot( 1, Nplots, iplot );
   imode = 1 + ( iplot - 1 ) * iskip;
   plot( real( Modes.phi( :, imode ) ), Modes.z )
   
   xlabel( [ 'Mode ' num2str( modes( imode ) ) ] )
   set( gca, 'YDir', 'Reverse' )   % because view messes up the zoom feature
   if ( iplot == 1 )
      ylabel( 'Depth (m)')
   else
      set( gca, 'YTickLabel', [' ';' '] ) % no tick lables
      %set( gca, 'YTickMode', 'manual')
   end
end

%figure; surf( modes, z, real( psi ) ); colorbar
