function scooterM( filename )

% Wavenumber integration code for ocean acoustics problems
% Based on the earlier Fortran version
% MBP Dec. 24, 2003

clear global
tic
global omega SSP Bdry
global NFirstAcoustic NLastAcoustic H

if ( isempty( filename ) )
    warndlg( 'No envfil has been selected', 'Warning' );
end

% filenames
envfil = [ filename '.env' ];   % input environmental file
shdfil = [ filename '.mat' ];   % output file name (pressure)
trcfil = filename;              % input top reflection coefficient
brcfil = filename;              % input bottom reflection coefficient

[ PlotTitle, freq, SSP, Bdry, Pos, Beam, cInt, RMax, fid ] = read_env( envfil, 'SCOOTER' );    % read in the environmental file
fclose( fid );                    % close out the envfil

PlotTitle = ['SCOOTER(M) -' PlotTitle ];

readrc( trcfil, brcfil, Bdry.Top.Opt(2:2), Bdry.Bot.Opt(1:1) );   % READ Reflection Coefficients (top and bottom)

omega  = 2 * pi * freq;

% Set up vector of wavenumber samples
kMin = omega / cInt.High;
kMax = omega / cInt.Low;

NkPts = floor( 1000.0 * RMax * ( kMax - kMin ) / pi );
fprintf( '\nNumber of wavenumbers, NkPts = %i \n', NkPts )

% Set-up the vector of k-space points
DeltaK = ( kMax - kMin ) / ( NkPts - 1 );
atten  = DeltaK;
rk     = linspace( kMin, kMax, NkPts );

H( 1:SSP.NMedia ) = ( SSP.depth( 2:SSP.NMedia + 1 ) - SSP.depth( 1:SSP.NMedia ) ) ./ SSP.N( 1:SSP.NMedia );   % vector of mesh widths
NptsAll       = sum( SSP.N( 1:SSP.NMedia ) ) + SSP.NMedia;   % number of solution points

[ B1, B2, B3, B4, rho ] = init_matrix( NptsAll );    % Initialize matrices

NptsAcoustic = sum( SSP.N( NFirstAcoustic:NLastAcoustic ) ) + 1;   % size of matrix for acoustic part
Green = kernel( B1, B2, B3, B4, rho, rk, atten, NptsAcoustic, Pos );
toc

PlotType    = 'rectilin  ';
Pos.Nrr   = NkPts;
Pos.r.range = rk;   % store wavenumbers is slot usually used for receiver ranges
% note p is 4-d: p( Ntheta, Nsd, Nrd, Nrr )
p( 1, 1:Pos.Nsd, 1:Pos.Nrd, 1:Pos.Nrr ) = Green;          % store Green's function in slot usually used for pressure
save GRNFIL PlotTitle PlotType freq atten Pos p
fieldsco( 'GRNFIL.mat' );
movefile( 'SHDFIL.mat', shdfil );
