from numpy import *
import time
import matplotlib.pyplot as plt
import pickle
from BellRead import *
from BellTrace import *

class BT:
	def __init__(self):
		self.ray = []
		self.Nsteps = 0
		self.omega  = 0
		self.Pos    = 0
		self.MxNarr = 0
		self.U      = 0

def bellhopP(filename):

	bt = BT
	
	starttime = time.clock()
	if (len(filename) == 0):
		disp('Warning: No envfil has been selected')
		exit()
	
	#global ray, Nsteps, omega, Bdry
	#global Pos
	#global alpha, Nbeams
	#global U, Arr, MxNarr
	# for optimization
	try:
		import psyco
		psyco.full()
	except ImportError:
		nothing = 0
	
	envfil = filename + '.env'    # input environmental file
	shdfil = filename + '.psh'    # output file name (pressure)
	arrfil = filename + '.psh'    # output file name (arrivals)
	btyfil = filename + '.bty'    # input bathymetry
	atifil = filename + '.ati'    # input altimetry
	sbpfil = filename + '.sbp'    # input source beam pattern
	trcfil = filename             # input top    reflection coefficient
	brcfil = filename             # input bottom reflection coefficient
	
	# read in the environmental file
	[ PlotTitle, freq, SSP, bt.Pos, Beam, cInt, RMax, fid, tr, br ] = read_env( envfil, 'BELLHOP' )    
	fid.close()      #close out the envfil
	Beam.Box.r  = 1000.0 * float(Beam.Box.r);
	bt.Pos.r.range = 1000 * bt.Pos.r.range; #convert km to m
	disp(PlotTitle) # used in saving the figure
	temp = PlotTitle
	PlotTitle = 'BELLHOP(P) -' + PlotTitle
	
	TopATI = tr.Bdry.Top.Opt[4]
	readati(atifil,TopATI,tr.Bdry.Top.depth,Beam.Box.r)
	
	BotBTY = tr.Bdry.Bot.Opt[1]
	readbty(btyfil,BotBTY,tr.Bdry.Bot.depth,Beam.Box.r)
	
	#READ Reflection Coefficients (top and bottom)
	readrc(trcfil, brcfil, tr.Bdry.Top.Opt[1], tr.Bdry.Bot.Opt[0])
	#optionally read source beam pattern
	SrcBmPat = readpat(sbpfil, Beam.RunType[2])	
	
	if ('A' == Beam.RunType[0].upper()):
		bt.MxNarr = round( max(2000000/(bt.Pos.Nrd*bt.Pos.Nrr), 10) )
		print('\n( Maximum # of arrivals = %i ) \n' % (bt.MxNarr))
		
		Arr = arr
		Arr.A     = zeros( (bt.Pos.Nrd, bt.Pos.Nrr, bt.MxNarr) )
		Arr.delay = zeros( (bt.Pos.Nrd, bt.Pos.Nrr, bt.MxNarr) )
		Arr.angle = zeros( (bt.Pos.Nrd, bt.Pos.Nrr, bt.MxNarr) )
		Arr.Narr  = zeros( (bt.Pos.Nrd, bt.Pos.Nrr) )
	else:
		p = zeros((1,bt.Pos.Nsd,bt.Pos.Nrd,bt.Pos.Nrr),complex)
	
	MaxSteps = 100000
	
	#allocate memory for "ray"
	bt.ray = []
	for jj in range(MaxSteps):
		bt.ray.append([[],[],[],[],[],[],[],[]])
	bt.omega = 2*pi*freq
	# calculate initial beam amplitude based on source beam pattern
	Amp0  = interp(br.alpha,SrcBmPat[:,0],SrcBmPat[:,1])
	
	br.alpha = (pi/180)*br.alpha # convert to radians
	Dalpha = 0.0
	if (br.Nbeams != 1):
		Dalpha = (br.alpha[br.Nbeams-1] - br.alpha[0])/(br.Nbeams-1)
		
	# Loop over source depths
	Layer = 1
	
	for s in range(int(bt.Pos.Nsd)):
		xs = array([0.0, bt.Pos.s.depth[s-1]], float) # source coordinate
		[ c, gradc, crr, crz, czz, Layer ] = ssp( xs, SSP, Layer )
		#RadMax = 10 * c / freq;  % 10 wavelength max radius
		
		#Are there enough beams??
		DalphaOpt = sqrt(c/(6.0*freq*bt.Pos.r.range[-1]))
		NbeamsOpt = round(2 + (br.alpha[br.Nbeams-1] - br.alpha[0])/DalphaOpt)
		
		if ( (Beam.RunType[0] == 'C') and (br.Nbeams < NbeamsOpt) ):
			print('Warning: Nbeams should be at least = %i \n' % (NbeamsOpt))
		
		# ****Trace rays*****
		bt.U = zeros(( len(bt.Pos.r.depth),len(bt.Pos.r.range) ),complex)
		if(Beam.RunType[0]=='R'):
			#plt.ion()
			plt.figure()
			plt.ylim(float(max(bt.Pos.r.depth)) + 1000.0, -1000.0)
			plt.title(PlotTitle)
		
		for ibeam in range(br.Nbeams):
			[bt.ray, bt.Nsteps, tr.Bdry] = trace(SSP,Beam.deltas,xs,br.alpha[ibeam],Amp0[ibeam],Beam.Type,Beam.Box,Beam.RunType,tr,bt)
			disp(' Tracing beam  %i'% ( ibeam + 1 ) )
			
			# ray trace run?
			if (Beam.RunType[0] == 'R'):
				
				r = zeros((bt.Nsteps,1))
				z = zeros((bt.Nsteps,1))
				for ii in range(bt.Nsteps):
					r[ii] = bt.ray[ii][1][0]
					z[ii] = bt.ray[ii][1][1]
				plt.plot(r,z,'b')
				
				#save RAYFIL ibeams Nsteps ray
			else:
				
				if (Beam.RunType[1] == 'G'):
					[bt.ray, bt.Nsteps, bt.omega, bt.Pos, bt.U] = influg(bt.Pos.s.depth[s], br.alpha[ibeam],Beam.RunType, Dalpha)
				else:
					[bt.ray, bt.Nsteps, bt.omega, bt.Pos, bt.U] = influgrb(bt.Pos.s.depth[s], br.alpha[ibeam], Beam.RunType, Dalpha,Beam.deltas)
		if (Beam.RunType[0] == 'A'):  #arrivals calculation
		
			#*****Thorpe attenuation?*******
			if (tr.Bdry.Top.Opt[3] == 'T'):
				f2 = (freq/1000.0)**2
				br.alpha = 40.0*f2/(4100.0 + f2) + 0.1*f2/(1.0 + f2)
				br.alpha = br.alpha/914.4  #dB/m
				br.alpha = br.alpha/8.6858896 #Nepers/m
			else:
				br.alpha = 0.0
			
			# add in attenuation
			for ir in range(bt.Pos.Nrr):
				factor = exp(-br.alpha*bt.Pos.r.range[ir])/sqrt(bt.Pos.r.range[ir])
				Arr.A[:,ir,:] = factor*Arr.A[:,ir,:]
			f = open(arrfil,'w')
			pickle.dump(PlotTitle, f)
			pickle.dump(bt.Pos,f)
			pickle.dump(Arr,f)
			pickle.dump(bt.MxNarr)
			f.close()
		else:
			p[0,s,:,:] = scalep(Dalpha,c,bt.Pos.r.range,bt.U,Beam.RunType,tr.Bdry.Top.Opt,freq)
			PlotType = 'rectilin  '
			atten = 0
			f = open(shdfil,'w')
			
			pickle.dump(PlotTitle,f)
			pickle.dump(PlotType,f)
			pickle.dump(freq,f)
			pickle.dump(atten,f)
			pickle.dump(p,f)
			pickle.dump(bt.Pos.r.depth,f)
			pickle.dump(bt.Pos.r.range,f)
			pickle.dump(bt.Pos.s.depth,f)
			pickle.dump(bt.Pos.Nrd,f)
			pickle.dump(bt.Pos.Nsd,f)
			pickle.dump(bt.Pos.Nrr,f)
			
			f.close()
			
			
		
	#next source depth
	#if(Beam.RunType[0]=='R'):
		#plt.ioff()
		#plt.savefig(temp)
	stoptime = time.clock()
	TIME = stoptime - starttime
	print('Elapsed time is %f seconds' % (TIME))
		