SUBROUTINE READIN( Title, Freq, MaxMedia, NMedia, &
     TopOpt, CPT, CST, rhoT, BumDen, eta, xi, NG, sigma, Depth, &
     BotOpt, CPB, CSB, rhoB, ENVFIL, PRTFIL )

  ! Reads in the info in ENVFIL

  IMPLICIT NONE
  INTEGER, PARAMETER :: MaxSSP = 2001
  INTEGER               ENVFIL, PRTFIL, NMedia, NG( * ), MaxMedia, NElts, Medium, Nneeded
  REAL    (KIND=8)   :: alphaR, betaR, alphaI, betaI, rhoR, Freq, rhoT, rhoB, &
                        sigma( * ), Depth( * ), BumDen, eta, xi, rho( 1 ), C, deltaz
  COMPLEX (KIND=8)   :: CPT, CST, CPB, CSB, CP( MaxSSP ), CS( MaxSSP )
  CHARACTER             TopOpt*( * ), BotOpt*( * ), Title*( * ), SSPType*1, AttenUnit*2, BCType*1, Task*8
  COMMON /CPREV/ alphaR, betaR, rhoR, alphaI, betaI

  ! OPEN ( FILE = 'ENVFIL', UNIT = ENVFIL, STATUS = 'OLD', FORM = 'FORMATTED' )

  alphaR = 1500.0
  betaR  = 0.0
  rhoR   = 1.0
  alphaI = 0.0
  betaI  = 0.0
  NElts  = 0         ! this is a dummy variable, passed to profil during read of SSP

  READ(  ENVFIL, *, END = 9999 ) Title(9:80)
  WRITE( PRTFIL, * ) Title

  READ( ENVFIL, *, END = 9999  ) Freq
  READ( ENVFIL, *, END = 9999  ) NMedia
  WRITE( PRTFIL, "( ' Frequency = ', G11.4, 'Hz   NMedia = ', I3, // )" ) Freq, NMedia

  IF ( NMedia > MaxMedia ) THEN
     WRITE( PRTFIL, * ) 'MaxMedia = ', MaxMedia
     CALL ERROUT( PRTFIL, 'F', 'READIN', 'Too many Media' )
  ENDIF

  ! TOP OPTIONS
  READ( ENVFIL, * ) TopOpt(1:8)
  SSPType         = TopOpt(1:1)
  BCType          = TopOpt(2:2)
  AttenUnit       = TopOpt(3:4)

  SELECT CASE ( SSPType )        ! SSP approximation options
  CASE ( 'N' )
     WRITE( PRTFIL, * ) '    N2-LINEAR approximation to SSP'
  CASE ( 'C' )
     WRITE( PRTFIL, * ) '    C-LINEAR approximation to SSP'
  CASE ( 'S' )
     WRITE( PRTFIL, * ) '    SPLINE approximation to SSP'
  CASE ( 'A' )
     WRITE( PRTFIL, * ) '    ANALYTIC SSP option'
  CASE DEFAULT
     CALL ERROUT( PRTFIL, 'F', 'READIN', 'Unknown option for SSP approximation' )
  END SELECT

  SELECT CASE ( AttenUnit(1:1) ) ! Attenuation options
  CASE ( 'N' )
     WRITE( PRTFIL, * ) '    Attenuation units: nepers/m'
  CASE ( 'F' )
     WRITE( PRTFIL, * ) '    Attenuation units: dB/mkHz'
  CASE ( 'M' ) 
     WRITE( PRTFIL, * ) '    Attenuation units: dB/m'
  CASE ( 'W' )
     WRITE( PRTFIL, * ) '    Attenuation units: dB/wavelength'
  CASE ( 'Q' )
     WRITE( PRTFIL, * ) '    Attenuation units: Q'
  CASE ( 'L' )
     WRITE( PRTFIL, * ) '    Attenuation units: Loss parameter'
  CASE DEFAULT
     CALL ERROUT( PRTFIL, 'F', 'READIN', 'Unknown attenuation units' )
  END SELECT

  SELECT CASE ( AttenUnit(2:2) ) !  Added volume attenuation
  CASE ( 'T' )
     WRITE( PRTFIL, * ) '    THORP attenuation added'
  END SELECT

  ! CALL TOPBOT  to read top BC 
  IF ( BCType == 'A' ) &
       WRITE( PRTFIL, "( //, '   Z (m)     alphaR (m/s)     betaR   rho (g/cm^3)  alphaI     betaI', / )" )
  CALL TOPBOT( ENVFIL, PRTFIL, Freq, BCType, AttenUnit, CPT, CST, rhoT, BumDen, eta, xi )

  !  Internal media 
  IF ( BCType /= 'A' ) &
       WRITE( PRTFIL, "( //, '   Z (m)     alphaR (m/s)     betaR   rho (g/cm^3)  alphaI     betaI', / )" )

  DO Medium = 1, NMedia
     READ(  ENVFIL, *, END = 9999 ) NG( Medium ), sigma( Medium ), Depth( Medium + 1 )
     WRITE( PRTFIL, "( /, '          ( Number of pts = ', I5, '  RMS roughness = ', G10.3, ' )')" ) &
          NG( Medium ),            sigma( Medium )

     !  Call PROFIL to read in SSP 
     Task = 'INIT'
     CALL PROFIL( Depth, CP, CS, rho, Medium, NElts, Freq, SSPType, AttenUnit, Task, ENVFIL, PRTFIL  )

     ! estimate number of points needed
     C = alphar
     IF ( betar > 0.0 ) C = betar     ! shear?
     deltaz = C / Freq / 20           ! default sampling: 20 points per wavelength
     Nneeded = ( Depth( Medium + 1 ) - Depth( Medium ) ) / deltaz
     Nneeded = MAX( Nneeded, 10 )     ! require a minimum of 10 points
  
     IF ( NG( Medium ) == 0 ) THEN         ! automatic calculation of f.d. mesh
        NG( Medium ) = Nneeded
        WRITE( PRTFIL, * ) 'Number of pts = ', NG( Medium )
     ELSEIF ( NG( Medium ) < Nneeded/2 ) THEN
       CALL ERROUT( PRTFIL, 'F', 'READIN', 'Mesh is too coarse' )
     END IF

  END DO   ! next Medium

  ! Bottom properties 
  READ( ENVFIL, *, END = 9999 ) BotOpt(1:8), sigma( NMedia + 1 )
  BCType = BotOpt(1:1)
  WRITE( PRTFIL, * )
  WRITE( PRTFIL, "( 33X, '( RMS roughness = ', G10.3, ' )' )" ) sigma( NMedia + 1 )

  ! CALL TOPBOT  to read bottom BC 
  CALL TOPBOT( ENVFIL, PRTFIL, Freq, BCType, AttenUnit, CPB, CSB, rhoB, BumDen, eta, xi )

  RETURN

9999 WRITE( PRTFIL, * ) 'Encountered end of file while reading ENVFIL'
  STOP

END SUBROUTINE READIN
!**********************************************************************!
SUBROUTINE TOPBOT( ENVFIL, PRTFIL, Freq, BCType, AttenUnit, CPHS, CSHS, rhoHS, BumDen, eta, xi )

  ! Handles top and bottom boundary conditions

  ! Input:
  !     ENVFIL: Environmental file
  !     PRTFIL: Print file
  !     Freq:   Frequency
  !     BCType: Boundary condition type
  !
  ! Output:
  !    CPHS:    P-wave speed in halfspace
  !    CSHS:    S-wave speed in halfspace
  !    rhoHS:   density in halfspace

  !    BumDen:  Bump density
  !    eta:     Principal radius 1
  !    xi:      Principal radius 2

  IMPLICIT NONE
  INTEGER             ENVFIL, PRTFIL
  REAL    (KIND=8) :: alphaR, betaR, alphaI, betaI, rhoR, Freq, rhoHS, BumDen, eta, xi, ZTMP
  COMPLEX (KIND=8) :: CPHS, CSHS, CRCI
  CHARACTER           BCType*1, AttenUnit*2
  COMMON /CPREV/ alphaR, betaR, rhoR, alphaI, betaI

  ! Echo to PRTFIL user's choice of boundary condition 
  SELECT CASE ( BCType )
  CASE ( 'S' )
     WRITE( PRTFIL, * ) '    Twersky SOFT BOSS scatter model'
  CASE ( 'H' )
     WRITE( PRTFIL, * ) '    Twersky HARD BOSS scatter model'
  CASE ( 'T' )
     WRITE( PRTFIL, * ) '    Twersky (amplitude only) SOFT BOSS scatter model'
  CASE ( 'I' )
     WRITE( PRTFIL, * ) '    Twersky (amplitude only) HARD BOSS scatter model'
  CASE ( 'V' )
     WRITE( PRTFIL, * ) '    VACUUM'
  CASE ( 'R' )
     WRITE( PRTFIL, * ) '    Perfectly RIGID'
  CASE ( 'A' )
     WRITE( PRTFIL, * ) '    ACOUSTO-ELASTIC half-space'
  CASE ( 'F' )
     WRITE( PRTFIL, * ) '    FILE used for reflection loss'
  CASE ( 'W' )
     WRITE( PRTFIL, * ) '    Writing an IRC file'
  CASE ( 'P' )
     WRITE( PRTFIL, * ) '    reading PRECALCULATED IRC'
  CASE DEFAULT
     CALL ERROUT( PRTFIL, 'F', 'TOPBOT', 'Unknown boundary condition type' )
  END SELECT

  ! Read in BC parameters depending on particular choice 
  CPHS  = 0.0
  CSHS  = 0.0
  rhoHS = 0.0

  ! Twersky ice model parameters 
  IF ( BCType == 'S' .OR. BCType == 'H' .OR. BCType == 'T' .OR. BCType == 'I' ) THEN
     READ(  ENVFIL, *    ) BumDen, eta, xi
     WRITE( PRTFIL, 1000 ) BumDen, eta, xi
1000 FORMAT( /, ' Twersky ice model parameters:', /, &
          ' Bumden = ', G15.6, '  Eta = ', G11.3, '  Xi = ', G11.3, /)
  ENDIF

  !  Half-space properties 
  IF ( BCType == 'A' ) THEN
     READ(  ENVFIL, *    ) ZTMP, alphaR, betaR, rhoR, alphaI, betaI
     WRITE( PRTFIL, 2000 ) ZTMP, alphaR, betaR, rhoR, alphaI, betaI
2000 FORMAT( F10.2, 3X, 2F10.2, 3X, F6.2, 3X, 2F10.4 )
     CPHS = CRCI( alphaR, alphaI, Freq, AttenUnit )
     CSHS = CRCI( betaR,  betaI,  Freq, AttenUnit )
     rhoHS = rhoR
     IF ( CPHS == 0.0 .OR. rhoHS == 0.0 ) &
          CALL ERROUT( PRTFil, 'F', 'TOPBOT', 'Sound speed or density vanishes in halfspace' )
  ENDIF

  RETURN
END SUBROUTINE TOPBOT
!**********************************************************************!
SUBROUTINE PROFIL( Depth, CP, CS, rhoT, Medium, N1, Freq, SSPType, AttenUnit, Task, ENVFIL, PRTFIL )

  ! Call the particular SSP routine specified by SSPType
  ! PROFIL is expected to perform two Tasks:
  !    Task = 'TAB'  then tabulate CP, CS, rhoT
  !    Task = 'INIT' then initialize
  ! Note that Freq is only need if Task = 'INIT'

  IMPLICIT NONE
  INTEGER             ENVFIL, PRTFIL, Medium, N1, I
  REAL    (KIND=8) :: rhoT( * ), Depth( * ), Freq, h, z
  COMPLEX (KIND=8) :: CPT, CST
  COMPLEX (KIND=8) :: CP( * ), CS( * )
  CHARACTER           SSPType*1, AttenUnit*2, Task*8

  SELECT CASE ( SSPType )
  CASE ( 'A' )  !  Analytic profile option 
     IF ( Task(1:4) == 'INIT' ) THEN
        N1 = 21
        CALL ANALYT( Depth, CP, CS, rhoT, Medium, N1, Freq, AttenUnit, Task )
        H = ( Depth( Medium+1 ) - Depth( Medium ) ) / ( N1 - 1 )

        DO I = 1, N1
           Z = Depth( Medium ) + ( I - 1 ) * H
           CPT =  CP( I )   ;    CST =  CS( I )
           WRITE( PRTFIL, 2000 ) Z,  REAL( CPT ),  REAL( CST ), rhoT(I), AIMAG( CPT ), AIMAG( CST )
2000       FORMAT( F10.2, 3X, 2F10.2, 3X, F6.2, 3X, 2F10.4 )
        END DO
     ELSE
        CALL ANALYT( Depth, CP, CS, rhoT, Medium, N1, Freq, AttenUnit, Task )
     ENDIF
  CASE ( 'N' )  !  N2-linear profile option 
     CALL N2LIN(  Depth, CP, CS, rhoT, Medium, N1, Freq, AttenUnit, Task, ENVFIL, PRTFIL )
  CASE ( 'C' )  !  C-linear profile option 
     CALL CLIN(   Depth, CP, CS, rhoT, Medium, N1, Freq, AttenUnit, Task, ENVFIL, PRTFIL )
  CASE ( 'S' )  !  Cubic spline profile option 
     CALL CCUBIC( Depth, CP, CS, rhoT, Medium, N1, Freq, AttenUnit, Task, ENVFIL, PRTFIL )
  CASE DEFAULT  !  Non-existent profile option 
     WRITE( PRTFIL, * ) 'Profile option: ', SSPType(1:1)
     CALL ERROUT( PRTFIL, 'F', 'PROFIL', 'Unknown profile option' )
  END SELECT

  RETURN
END SUBROUTINE PROFIL
!**********************************************************************!
FUNCTION CRCI( C, alpha, Freq, AttenUnit )

  ! Converts real wave speed and attenuation to a single
  !  complex wave speed (with positive imaginary part)

  ! 6 CASES:    N for Nepers/meter
  !             M for dB/meter      (M for Meters)
  !             F for dB/m-kHZ      (F for Frequency dependent)
  !             W for dB/wavelength (W for Wavelength)
  !             Q for Q
  !             L for Loss parameter
  !
  ! second letter adds volume attenuation according to standard laws:
  !             T for Thorp

  IMPLICIT NONE
  REAL (KIND=8) :: PI
  PARAMETER ( PI = 3.1415926535897932D0 )
  REAL    (KIND=8) :: Freq, F2, Omega, alpha, alphaT, C, Thorpe
  COMPLEX (KIND=8) :: CRCI
  CHARACTER           AttenUnit*2

  Omega = 2.0 * PI * Freq

  !  Convert to Nepers/m 
  alphaT = 0.0
  SELECT CASE ( AttenUnit(1:1) )
  CASE ( 'N' )
     alphaT = alpha
  CASE ( 'M' )
     alphaT = alpha / 8.6858896D0
  CASE ( 'F' )
     alphaT = alpha * Freq / 8685.8896D0
  CASE ( 'W' )
     IF ( C /= 0.0 ) alphaT = alpha * Freq / ( 8.6858896D0 * C )
     !        The following lines give f^1.25 Frequency dependence
     !        FAC = SQRT( SQRT( Freq / 50.0 ) )
     !        IF ( C /= 0.0 ) alphaT = FAC * alpha * Freq / ( 8.6858896D0 * C )
  CASE ( 'Q' )
     IF( C * alpha /= 0.0 ) alphaT = Omega / ( 2.0 * C * alpha )
  CASE ( 'L' )   ! loss parameter
     IF ( C /= 0.0 ) alphaT = alpha * Omega / C
  END SELECT

  ! added volume attenuation
  SELECT CASE ( AttenUnit(2:2) )
  CASE ( 'T' )
     F2 = ( Freq / 1000.0 ) **2
     Thorpe = 40.0 * F2 / ( 4100.0 + F2 ) + 0.1 * F2 / ( 1.0 + F2 )
     Thorpe = Thorpe / 914.4D0                 ! dB / m
     Thorpe = Thorpe / 8.6858896D0             ! Nepers / m
     alphaT = alphaT + Thorpe
  END SELECT

  ! Convert Nepers/m to equivalent imaginary sound speed 
  alphaT = alphaT * C * C / Omega
  CRCI = CMPLX( C, alphaT, KIND=8 )

  RETURN
END FUNCTION CRCI
!**********************************************************************!
SUBROUTINE N2LIN( Depth, CP, CS, rhoT, Medium, N1, Freq, AttenUnit, Task, ENVFIL, PRTFIL )

  ! Tabulate CP, CS, rho for specified Medium
  ! Uses N2-linear segments for P and S-wave speeds
  ! Uses rho-linear segments for density

  IMPLICIT NONE
  INTEGER ENVFIL, PRTFIL, MaxMedia, MaxSSP
  PARAMETER ( MaxMedia = 501, MaxSSP = 2001 )
  INTEGER             Loc( MaxMedia ), NSSPPts( MaxMedia ), Medium, N, N1, I, ILoc, Lay
  REAL    (KIND=8) :: Depth( * ), rhoT( * ), Z( MaxSSP ), rho( MaxSSP ), Freq, alphaR, betaR, rhoR, alphaI, betaI, H, R, ZT
  COMPLEX (KIND=8) :: CP( * ), CS( * ), alpha( MaxSSP ), beta( MaxSSP ), N2BOT, N2TOP, CRCI
  CHARACTER           AttenUnit*2, Task*8
  SAVE Z, alpha, beta, rho, Loc, NSSPPts
  COMMON /CPREV/ alphaR, betaR, rhoR, alphaI, betaI

  ! If Task = 'INIT' then this is the first call and SSP is read.
  ! Any other call is a request for SSP subtabulation.

  IF ( Task(1:4) == 'INIT' ) THEN   ! Task 'INIT' for initialization

     ! The variable Loc( Medium ) points to the starting point for the
     ! data in the arrays Z, alpha, beta and rho
     IF ( Medium == 1 ) THEN
        Loc( Medium ) = 0
     ELSE
        Loc( Medium ) = Loc( Medium - 1 ) + NSSPPts( Medium - 1 )
     ENDIF
     ILoc = Loc( Medium )

     !  Read in data and convert attenuation to Nepers/m 
     N1 = 1
     DO I = 1, MaxSSP
        READ(  ENVFIL, *    ) Z( ILoc + I ), alphaR, betaR, rhoR, alphaI, betaI
        WRITE( PRTFIL, 2000 ) Z( ILoc + I ), alphaR, betaR, rhoR, alphaI, betaI
2000    FORMAT( F10.2, 3X, 2F10.2, 3X, F6.2, 3X, 2F10.4 )

        alpha(ILoc + I) = CRCI( alphaR, alphaI, Freq, AttenUnit )
        beta( ILoc + I) = CRCI( betaR,  betaI,  Freq, AttenUnit )
        rho(  ILoc + I) = rhoR

        ! Did we read the last point?
        IF ( ABS( Z( ILoc + I ) - Depth( Medium+1 ) ) < EPSILON( 1.0e0 ) * Depth( Medium+1 ) ) THEN
           NSSPPts( Medium ) = N1
           IF ( Medium == 1 ) Depth( 1 ) = Z( 1 )
           RETURN
        ENDIF

        N1 = N1 + 1
     END DO

     ! Fall through means too many points in the profile
     WRITE( PRTFIL, * ) 'Max. #SSP points: ', MaxSSP
     CALL ERROUT( PRTFIL, 'F', 'N2LIN', 'Number of SSP points exceeds limit' )

  ELSE   ! Task = 'TABULATE'
     ILoc = Loc( Medium )
     N    = N1 - 1
     H    = ( Z( ILoc + NSSPPts( Medium ) ) - Z( ILoc + 1 ) ) / N
     Lay  = 1

     DO I = 1, N1
        ZT = Z( ILoc + 1 ) + ( I - 1 ) * H
        IF ( I == N1 ) ZT = Z( ILoc + NSSPPts( Medium ) )   ! Make sure no overshoot

        DO WHILE ( ZT > Z( ILoc + Lay + 1 ) )
           Lay = Lay + 1
        END DO

        R = ( ZT - Z( ILoc + Lay ) ) / ( Z( ILoc + Lay+1 ) - Z( ILoc + Lay ) )

        ! P-wave
        N2TOP   = 1.0 / alpha( ILoc + Lay     )**2
        N2BOT   = 1.0 / alpha( ILoc + Lay + 1 )**2
        CP( I ) = 1.0 / SQRT( ( 1.0 - R ) * N2TOP + R * N2BOT )

        ! S-wave
        IF ( beta(ILoc + Lay) /= 0.0 ) THEN
           N2TOP   = 1.0 / beta( ILoc + Lay     )**2
           N2BOT   = 1.0 / beta( ILoc + Lay + 1 )**2
           CS( I ) = 1.0 / SQRT( ( 1.0 - R ) * N2TOP + R * N2BOT )
        ELSE
           CS( I ) = 0.0
        ENDIF

        rhoT( I ) = ( 1.0 - R ) * rho( ILoc + Lay ) + R * rho( ILoc + Lay + 1 )
     END DO

  ENDIF

  RETURN
END SUBROUTINE N2LIN
!**********************************************************************!
SUBROUTINE CLIN( Depth, CP, CS, rhoT, Medium, N1, Freq, AttenUnit, Task, ENVFIL, PRTFIL  )

  ! Tabulate CP, CS, rho for specified Medium

  ! Uses c-linear segments for P and S-wave speeds
  ! Uses rho-linear segments for density
  IMPLICIT NONE
  INTEGER ENVFIL, PRTFIL, MaxMedia, MaxSSP
  PARAMETER ( MaxMedia = 501, MaxSSP = 2001 )
  INTEGER             Loc( MaxMedia ), NSSPPts( MaxMedia ), Medium, N, N1, I, ILoc, Lay
  REAL    (KIND=8) :: Depth( * ), rhoT( * ), Z( MaxSSP ), rho( MaxSSP ), Freq, alphaR, betaR, rhoR, alphaI, betaI, H, R, ZT
  COMPLEX (KIND=8) :: CP( * ), CS( * ), alpha( MaxSSP ), beta( MaxSSP ), CRCI
  CHARACTER           AttenUnit*2, Task*8
  SAVE Z, alpha, beta, rho, Loc, NSSPPts
  COMMON /CPREV/ alphaR, betaR, rhoR, alphaI, betaI

  ! If Task = 'INIT' then this is the first call and SSP is read.
  ! Any other call is a request for SSP subtabulation.

  IF ( Task(1:4) == 'INIT' ) THEN   ! Task 'INIT' FOR INITIALIZATION
     NSSPPts( Medium ) = N1

     ! The variable Loc(Medium) points to the starting point for the
     ! data in the arrays Z, alpha, beta and rho

     IF ( Medium == 1 ) THEN
        Loc( Medium ) = 0
     ELSE
        Loc( Medium ) = Loc( Medium - 1 ) + NSSPPts( Medium - 1 )
     ENDIF
     ILoc = Loc( Medium )

     !  Read in data and convert attenuation to Nepers/m 
     N1 = 1
     DO I = 1, MaxSSP
        READ(  ENVFIL, *    ) Z( ILoc + I ), alphaR, betaR, rhoR, alphaI, betaI
        WRITE( PRTFIL, 2000 ) Z( ILoc + I ), alphaR, betaR, rhoR, alphaI, betaI
2000    FORMAT( F10.2, 3X, 2F10.2, 3X, F6.2, 3X, 2F10.4 )

        alpha( ILoc + I ) = CRCI( alphaR, alphaI, Freq, AttenUnit )
        beta(  ILoc + I ) = CRCI( betaR,  betaI,  Freq, AttenUnit )
        rho(   ILoc + I ) = rhoR

        ! Did we read the last point?
        IF ( ABS( Z( ILoc + I ) - Depth( Medium+1 ) ) <  EPSILON( 1.0e0 ) * Depth( Medium+1 ) ) THEN
           NSSPPts( Medium ) = N1
           IF ( Medium == 1 ) Depth( 1 ) = Z( 1 )
           RETURN
        ENDIF

        N1 = N1 + 1
     END DO

     ! Fall through means too many points in the profile

     WRITE( PRTFIL, * ) 'Max. #SSP points: ', MaxSSP
     CALL ERROUT( PRTFIL, 'F', 'CLIN', 'Number of SSP points exceeds limit' )

  ELSE   ! Task = 'TABULATE'
     ILoc = Loc( Medium )
     N    = N1 - 1
     H    = ( Z( ILoc + NSSPPts( Medium ) ) - Z( ILoc + 1 ) ) / N
     Lay  = 1

     DO I = 1, N1
        ZT = Z( ILoc + 1 ) + ( I - 1 ) * H
        IF ( I == N1 ) ZT = Z( ILoc + NSSPPts( Medium ) )   ! Make sure no overshoot

        DO WHILE ( ZT > Z( ILoc + Lay + 1 ) )
           Lay = Lay + 1
        END DO

        R = ( ZT - Z( ILoc + Lay ) ) / ( Z( ILoc + Lay + 1 ) - Z( ILoc + Lay ) )
        CP(   I ) = ( 1.0 - R ) * alpha( ILoc + Lay ) + R * alpha( ILoc + Lay+1 )
        CS(   I ) = ( 1.0 - R ) *  beta( ILoc + Lay ) + R *  beta( ILoc + Lay+1 )
        rhoT( I ) = ( 1.0 - R ) *   rho( ILoc + Lay ) + R *   rho( ILoc + Lay+1 )
     END DO
  ENDIF

  RETURN
END SUBROUTINE CLIN
!**********************************************************************!
SUBROUTINE CCUBIC( Depth, CP, CS, rhoT, Medium, N1, Freq, AttenUnit, Task, ENVFIL, PRTFIL  )

  ! Tabulate CP, CS, rho for specified Medium
  ! using cubic spline interpolation

  IMPLICIT NONE
  INTEGER ENVFIL, PRTFIL, MaxMedia, MaxSSP
  PARAMETER ( MaxMedia = 501, MaxSSP = 2001 )
  INTEGER             Loc( MaxMedia ), NSSPPts( MaxMedia ), Medium, N, N1, I, ILoc, Lay, IBCBeg, IBCEnd
  REAL    (KIND=8) :: Depth( * ), rhoT( * ), Z( MaxSSP ), Freq, alphaR, betaR, rhoR, alphaI, betaI, h, HSPLNE, ZT
  COMPLEX (KIND=8) :: CP( * ), CS( * ), ESPLINE, CRCI, alpha( 4, MaxSSP ), beta( 4, MaxSSP ), rho( 4, MaxSSP )
  CHARACTER           AttenUnit*2, Task*8

  SAVE Z, alpha, beta, rho, Loc, NSSPPts
  COMMON /CPREV/ alphaR, betaR, rhoR, alphaI, betaI

  ! If Task = 'INIT' then this is the first call and SSP is read.
  ! Any other call is a request for SSP subtabulation.

  IF ( Task(1:4) == 'INIT' ) THEN   ! --- Task 'INIT' for initialization
     NSSPPts( Medium ) = N1

     ! The variable Loc(Medium) points to the starting point for the
     ! data in the arrays Z, alpha, beta and rho

     IF ( Medium == 1 ) THEN
        Loc( Medium ) = 0
     ELSE
        Loc( Medium ) = Loc( Medium - 1 ) + NSSPPts( Medium - 1 )
     ENDIF
     ILoc = Loc( Medium )

     !  Read in data and convert attenuation to Nepers/m 
     N1 = 1

     DO I = 1, MaxSSP
        READ(  ENVFIL, *    ) Z( ILoc + I ), alphaR, betaR, rhoR, alphaI, betaI
        WRITE( PRTFIL, 2000 ) Z( ILoc + I ), alphaR, betaR, rhoR, alphaI, betaI
2000    FORMAT(F10.2, 3X, 2F10.2, 3X, F6.2, 3X, 2F10.4)

        alpha(1, ILoc + I) = CRCI( alphaR, alphaI, Freq, AttenUnit )
        beta( 1, ILoc + I) = CRCI( betaR,  betaI,  Freq, AttenUnit )
        rho(  1, ILoc + I) = rhoR

        ! Did we read the last point?
        IF ( ABS( Z( ILoc + I ) - Depth(Medium+1) ) <  EPSILON( 1.0e0 ) * Depth( Medium+1 ) ) THEN
           NSSPPts( Medium ) = N1
           IF ( Medium == 1 ) Depth( 1 ) = Z( 1 )

           !   Compute spline coefs 
           IBCBEG = 0
           IBCEND = 0
           CALL CSPLINE( Z( ILoc + 1 ), alpha( 1, ILoc + 1 ), NSSPPts( Medium ), IBCBEG, IBCEND, NSSPPts( Medium ) )
           CALL CSPLINE( Z( ILoc + 1 ),  beta( 1, ILoc + 1 ), NSSPPts( Medium ), IBCBEG, IBCEND, NSSPPts( Medium ) )
           CALL CSPLINE( Z( ILoc + 1 ),   rho( 1, ILoc + 1 ), NSSPPts( Medium ), IBCBEG, IBCEND, NSSPPts( Medium ) )

           RETURN
        ENDIF

        N1 = N1 + 1
     END DO

     ! Fall through means too many points in the profile

     WRITE( PRTFIL, * ) 'Max. #SSP points: ', MaxSSP
     CALL ERROUT( PRTFIL, 'F', 'CCUBIC', 'Number of SSP points exceeds limit' )

  ELSE   ! Task = 'TABULATE'
     ILoc = Loc( Medium )
     N    = N1 - 1
     H    = ( Z( ILoc + NSSPPts( Medium ) ) - Z( ILoc + 1 ) ) / N
     Lay  = 1
     DO I = 1, N1
        ZT = Z( ILoc + 1 ) + ( I - 1 ) * H
        IF ( I == N1 ) ZT = Z( ILoc + NSSPPts( Medium ) )   ! Make sure no overshoot
        DO WHILE ( ZT > Z( ILoc + Lay + 1 ) )
           Lay = Lay + 1
        END DO

        HSPLNE = ZT - Z( ILoc + Lay )

        CP(   I ) =       ESPLINE( alpha( 1, ILoc + Lay ), HSPLNE )
        CS(   I ) =       ESPLINE(  beta( 1, ILoc + Lay ), HSPLNE )
        rhoT( I ) = DBLE( ESPLINE(   rho( 1, ILoc + Lay ), HSPLNE ) )

     END DO
  ENDIF

  RETURN
END SUBROUTINE CCUBIC
